SimpleSelfRebuff:_rev("$Revision: 49781 $")

-------------------------------------------------------------------------------
-- Externals
-------------------------------------------------------------------------------

local SimpleSelfRebuff = SimpleSelfRebuff
local L = AceLibrary("AceLocale-2.2"):new("SimpleSelfRebuff")

local STATE_UNSET   = SimpleSelfRebuff.STATE_UNSET
local STATE_BUFFED  = SimpleSelfRebuff.STATE_BUFFED
local STATE_FADING  = SimpleSelfRebuff.STATE_FADING
local STATE_MISSING = SimpleSelfRebuff.STATE_MISSING

local new = SimpleSelfRebuff.new
local del = SimpleSelfRebuff.del

-------------------------------------------------------------------------------
-- Locals
-------------------------------------------------------------------------------

local db

local bindingButton
local TEH_BUTTON_NAME = "SimpleSelfRebuff_MouseWheelBindingButton"

local pendingBuff
local boundBuff

local boundKeys = {}

local castCooldown

local updateEvents = {
	'SimpleSelfRebuff_MonitoringEnabled',
	'SimpleSelfRebuff_StateChanged',
	'PLAYER_CONTROL_GAINED',
	'PLAYER_REGEN_ENABLED',
	'PLAYER_AURAS_CHANGED'
}

-------------------------------------------------------------------------------
-- Module declaration
-------------------------------------------------------------------------------

local MouseWheelBinding = SimpleSelfRebuff:NewModule('MouseWheelBinding')

MouseWheelBinding.optionName = L["Mouse wheel bindings"]

MouseWheelBinding.options = {
	zoom = {
		type = 'toggle',
		name = L["Zoom when casting"],
		desc = L["Enable zooming when using the mousewheel to cast."],
		get = function() return db.zoom end,
		set = function(v) db.zoom = v end
	},
}

function MouseWheelBinding:OnModuleEnable()
	self:RegisterDefaults('profile', {
		zoom = true,
	})
	db = self.db.profile

	self:RegisterBucketEvent(updateEvents, 0.1, 'UpdateBinding')
	
	self:RegisterEvent('PLAYER_REGEN_DISABLED', 'SetBinding', false)
	self:RegisterEvent('SimpleSelfRebuff_MonitoringDisabled', 'SetBinding', false)
	self:RegisterEvent('UPDATE_BINDINGS', 'KeyBindingChanged')
	self:KeyBindingChanged()
		
	self:UpdateBinding()
end

function MouseWheelBinding:OnModuleDisable()
	self:SetBinding(false)
end

function MouseWheelBinding:KeyBindingChanged()
	boundKeys = del(boundKeys)
	boundKeys = new("MOUSEWHEELUP", "MOUSEWHEELDOWN", GetBindingKey('SIMPLE_REBUFF'))
end

function MouseWheelBinding:UpdateBinding()
	if self.core:IsMonitoringActive() then
		pendingBuff = self.core:GetPendingBuff(pendingBuff)	
		self:SetBinding(pendingBuff)		
	else
		self:SetBinding(false)
	end
end

function MouseWheelBinding:SetBinding(buff)

	if buff then
		
		if InCombatLockdown() or buff == boundBuff then
			return
		end
		
		if not bindingButton then
			bindingButton = CreateFrame("Button", TEH_BUTTON_NAME, UIParent, "SecureActionButtonTemplate")
			bindingButton:RegisterForClicks("LeftButtonUp")
			bindingButton:SetPoint("CENTER",0,0)
			bindingButton:SetScript("PreClick", function(...) return self:OnButtonPreClick(...) end)
			bindingButton:SetScript("PostClick", function(...) return self:OnButtonPostClick(...) end)
			bindingButton:Hide()
			
			self:ClearButtonBindings()
		end
		
		-- Binding
		self.core:Debug('Binding mousewheel for %q', buff.name)
		for i,key in ipairs(boundKeys) do
			SetOverrideBindingClick(bindingButton, true, key, TEH_BUTTON_NAME, key)
		end
		
		boundBuff = buff
		
	elseif boundBuff then
		
		if InCombatLockdown() then
			self:ScheduleLeaveCombatAction('SetBinding', false)
		else		
			self.core:Debug('Unbinding mousewheel')
			self:ClearButtonBindings()
			ClearOverrideBindings(bindingButton)
			boundBuff = nil
		end
		
	end

end

function MouseWheelBinding:OnButtonPreClick()
	if not InCombatLockdown() then
		if castCooldown and castCooldown > GetTime() then
			return
		end
		self:ClearButtonBindings()
		if boundBuff and not boundBuff:IsInCooldown() then
			boundBuff:SetupSecureButton(bindingButton)
		end
	end
end

function MouseWheelBinding:OnButtonPostClick(button, key)
	if db.zoom then
		if key == 'MOUSEWHEELUP' then
			CameraZoomIn(1)
		elseif key == 'MOUSEWHEELDOWN' then
			CameraZoomOut(1)
		end
	end

	if boundBuff and not InCombatLockdown() then
		castCooldown = GetTime() + 2
		self:ClearButtonBindings()
	end
end

function MouseWheelBinding:ClearButtonBindings()
	if bindingButton then
		bindingButton:SetAttribute('type', nil)
		bindingButton:SetAttribute('spell', nil)			
		bindingButton:SetAttribute('unit', nil)			
		bindingButton:SetAttribute('target-slot', nil)					
	end
end

function MouseWheelBinding:DummyBinding()
	self:Debug("Do nothing")
end
